@@ -59,7 +59,7 @@ struct _GDaemonVfsClass
   GVfsClass parent_class;
 };
 
-G_DEFINE_DYNAMIC_TYPE (GDaemonVfs, g_daemon_vfs, G_TYPE_VFS);
+G_DEFINE_DYNAMIC_TYPE (GDaemonVfs, g_daemon_vfs, G_TYPE_VFS)
 
 static GDaemonVfs *the_vfs = NULL;
 
@@ -106,7 +106,7 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
   char *scheme;
   GVfsUriMountInfo *info;
   
-  scheme = g_uri_get_scheme (uri);
+  scheme = g_uri_parse_scheme (uri);
   if (scheme == NULL)
     return FALSE;
   
@@ -122,7 +122,7 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
 	{
 	  spec = g_mount_spec_new_from_data (info->keys, NULL);
 	  path = info->path;
-	  /* We took over ownership, custom free: */
+	  /* We took over ownership of info parts, custom free: */
 	  g_free (info);
 	}
     }
@@ -175,7 +175,10 @@ g_daemon_vfs_init (GDaemonVfs *vfs)
   GVfsUriMapper *mapper;
   GList *modules;
   int i;
-  
+
+  bindtextdomain (GETTEXT_PACKAGE, GVFS_LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
   vfs->async_bus = dbus_bus_get_private (DBUS_BUS_SESSION, NULL);
 
   if (vfs->async_bus == NULL)
@@ -282,6 +285,43 @@ g_daemon_vfs_get_file_for_uri (GVfs       *vfs,
   return g_vfs_get_file_for_uri (G_DAEMON_VFS (vfs)->wrapped_vfs, uri);
 }
 
+GMountSpec *
+_g_daemon_vfs_get_mount_spec_for_path (GMountSpec *spec,
+				       const char *path,
+				       const char *new_path)
+{
+  const char *type;
+  GVfsUriMapper *mapper;
+  GMountSpec *new_spec;
+
+  type = g_mount_spec_get_type (spec);
+
+  if (type == NULL)
+    return g_mount_spec_ref (spec);
+  
+  new_spec = NULL;
+  mapper = g_hash_table_lookup (the_vfs->to_uri_hash, type);
+  if (mapper)
+    {
+      GVfsUriMountInfo info, *new_info;
+      info.keys = spec->items;
+      info.path = (char *)path;
+      new_info = g_vfs_uri_mapper_get_mount_info_for_path (mapper, &info, new_path);
+      if (new_info != NULL)
+	{
+	  new_spec = g_mount_spec_new_from_data (new_info->keys, NULL);
+	  /* We took over ownership of parts of new_info, custom free: */
+	  g_free (new_info->path);
+	  g_free (new_info);
+	}
+    }
+
+  if (new_spec == NULL)
+    new_spec = g_mount_spec_ref (spec);
+
+  return new_spec;
+}
+
 char *
 _g_daemon_vfs_get_uri_for_mountspec (GMountSpec *spec,
 				     char *path,
@@ -521,11 +561,9 @@ lookup_mount_info_in_cache (GMountSpec *spec,
 void
 _g_daemon_vfs_invalidate_dbus_id (const char *dbus_id)
 {
-  GMountInfo *info;
   GList *l, *next;
 
   G_LOCK (mount_cache);
-  info = NULL;
   for (l = the_vfs->mount_cache; l != NULL; l = next)
     {
       GMountInfo *mount_info = l->data;
@@ -769,9 +807,6 @@ g_daemon_vfs_class_init (GDaemonVfsClass *class)
 
   vfs_class = G_VFS_CLASS (class);
 
-  vfs_class->name = "gvfs";
-  vfs_class->priority = 10;
-  
   vfs_class->is_active = g_daemon_vfs_is_active;
   vfs_class->get_file_for_path = g_daemon_vfs_get_file_for_path;
   vfs_class->get_file_for_uri = g_daemon_vfs_get_file_for_uri;
@@ -791,6 +826,11 @@ g_io_module_load (GIOModule *module)
   g_daemon_vfs_register_type (G_TYPE_MODULE (module));
   g_daemon_volume_monitor_register_types (G_TYPE_MODULE (module));
 
+  g_io_extension_point_implement (G_VFS_EXTENSION_POINT_NAME,
+				  G_TYPE_DAEMON_VFS,
+				  "gvfs",
+				  10);
+  
   g_vfs_uri_mapper_register (module);
   g_vfs_uri_mapper_smb_register (module);
   g_vfs_uri_mapper_sftp_register (module);
