@@ -47,6 +47,14 @@
 #include "gvfsjobqueryattributes.h"
 #include "gvfsjobenumerate.h"
 #include "gvfsdaemonprotocol.h"
+#include "gvfskeyring.h"
+
+#ifdef HAVE_GCONF
+#include <gconf/gconf-client.h>
+#endif
+
+/* We load a default workgroup from gconf */
+#define PATH_GCONF_GNOME_VFS_SMB_WORKGROUP "/system/smb/workgroup"
 
 #include <libsmbclient.h>
 
@@ -68,6 +76,9 @@ struct _GVfsBackendSmb
   GMountSource *mount_source; /* Only used/set during mount */
   int mount_try;
   gboolean mount_try_again;
+	
+  gboolean password_in_keyring;
+  GPasswordSave password_save;
   
   /* Cache */
   char *cached_server_name;
@@ -77,7 +88,9 @@ struct _GVfsBackendSmb
   SMBCSRV *cached_server;
 };
 
-G_DEFINE_TYPE (GVfsBackendSmb, g_vfs_backend_smb, G_VFS_TYPE_BACKEND);
+static char *default_workgroup = NULL;
+
+G_DEFINE_TYPE (GVfsBackendSmb, g_vfs_backend_smb, G_VFS_TYPE_BACKEND)
 
 static void
 g_vfs_backend_smb_finalize (GObject *object)
@@ -168,35 +181,58 @@ auth_callback (SMBCCTX *context,
     }
   else
     {
-      GAskPasswordFlags flags = G_ASK_PASSWORD_NEED_PASSWORD;
-      char *message;
-      
-      if (backend->domain == NULL)
-	flags |= G_ASK_PASSWORD_NEED_DOMAIN;
-      if (backend->user == NULL)
-	flags |= G_ASK_PASSWORD_NEED_USERNAME;
+      gboolean in_keyring = FALSE;
 
-      /* translators: First %s is a share name, second is a server name */
-      message = g_strdup_printf (_("Password required for share %s on %s"),
-				 share_name, server_name);
-      handled = g_mount_source_ask_password (backend->mount_source,
-					     message,
-					     username_out,
-					     domain_out,
-					     flags,
-					     &abort,
-					     &ask_password,
-					     &ask_user,
-					     &ask_domain);
-      g_free (message);
-      if (!handled)
-	goto out;
+      if (!backend->password_in_keyring)
+        {
+	  in_keyring = g_vfs_keyring_lookup_password (backend->user,
+						      backend->server,
+						      backend->domain,
+						      NULL,
+						      NULL,
+						      0,
+						      "smb",
+						      &ask_user,
+						      &ask_domain,
+						      &ask_password);
+	  backend->password_in_keyring = in_keyring;
+	}
       
-      if (abort)
-	{
-	  strncpy (username_out, "ABORT", unmaxlen);
-	  strncpy (password_out, "", pwmaxlen);
-	  goto out;
+      if (!in_keyring)
+        {
+	  GAskPasswordFlags flags = G_ASK_PASSWORD_NEED_PASSWORD;
+	  char *message;
+      
+	  if (g_vfs_keyring_is_available ())
+	    flags |= G_ASK_PASSWORD_SAVING_SUPPORTED;
+	  if (backend->domain == NULL)
+	    flags |= G_ASK_PASSWORD_NEED_DOMAIN;
+	  if (backend->user == NULL)
+	    flags |= G_ASK_PASSWORD_NEED_USERNAME;
+
+	  /* translators: First %s is a share name, second is a server name */
+	  message = g_strdup_printf (_("Password required for share %s on %s"),
+				     share_name, server_name);
+	  handled = g_mount_source_ask_password (backend->mount_source,
+						 message,
+						 username_out,
+						 domain_out,
+						 flags,
+						 &abort,
+						 &ask_password,
+						 &ask_user,
+						 &ask_domain,
+						 &(backend->password_save));
+	  g_free (message);
+	  if (!handled)
+	    goto out;
+      
+	  if (abort)
+	    {
+	      strncpy (username_out, "ABORT", unmaxlen);
+	      strncpy (password_out, "", pwmaxlen);
+	      goto out;
+	    }
 	}
 
       /* Try again if this fails */
@@ -441,6 +477,11 @@ do_mount (GVfsBackend *backend,
   smb_context->callbacks.get_cached_srv_fn    = get_cached_server;
   smb_context->callbacks.remove_cached_srv_fn = remove_cached_server;
   smb_context->callbacks.purge_cached_fn      = purge_cached;
+ 
+  /* libsmbclient frees this on it's own, so make sure 
+   * to use simple system malloc */
+  if (default_workgroup != NULL)
+    smb_context->workgroup = strdup (default_workgroup);
   
   smb_context->flags = 0;
   
@@ -489,6 +530,7 @@ do_mount (GVfsBackend *backend,
 
   op_backend->mount_source = mount_source;
   op_backend->mount_try = 0;
+  op_backend->password_save = G_PASSWORD_SAVE_NEVER;
 
   do
     {
@@ -518,6 +560,16 @@ do_mount (GVfsBackend *backend,
 			_("Failed to mount Windows share"));
       return;
     }
+
+  g_vfs_keyring_save_password (op_backend->last_user,
+			       op_backend->server,
+			       op_backend->last_domain,
+			       "smb",
+			       NULL,
+			       NULL,
+			       0,
+			       op_backend->last_password,
+			       op_backend->password_save);
   
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
@@ -587,6 +639,12 @@ do_read (GVfsBackend *backend,
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   ssize_t res;
 
+  /* For some reason requests of 65536 bytes broke for me (returned 0)
+   * Maybe some smb protocol limit
+   */
+  if (bytes_requested > 65535)
+    bytes_requested = 65535;
+  
   res = op_backend->smb_context->read (op_backend->smb_context, (SMBCFILE *)handle, buffer, bytes_requested);
 
   if (res == -1)
@@ -898,9 +956,11 @@ do_replace (GVfsBackend *backend,
 					O_CREAT|O_WRONLY|O_EXCL, 0);
   if (file == NULL && errno != EEXIST)
     {
+      int errsv = errno;
+
       g_set_error (&error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
-		   g_strerror (errno));
+		   g_io_error_from_errno (errsv),
+		   "%s", g_strerror (errsv));
       goto error;
     }
   else if (file == NULL && errno == EEXIST)
@@ -918,7 +978,8 @@ do_replace (GVfsBackend *backend,
 		  g_set_error (&error,
 			       G_IO_ERROR,
 			       G_IO_ERROR_WRONG_ETAG,
-			       _("The file was externally modified"));
+			       "%s",
+                               _("The file was externally modified"));
 		  goto error;
 		}
 	      g_free (current_etag);
@@ -949,11 +1010,13 @@ do_replace (GVfsBackend *backend,
 		    g_set_error (&error,
 				 G_IO_ERROR,
 				 G_IO_ERROR_CANCELLED,
+                                 "%s",
 				 _("Operation was cancelled"));
 		  else
 		    g_set_error (&error,
 				 G_IO_ERROR,
 				 G_IO_ERROR_CANT_CREATE_BACKUP,
+                                 "%s",
 				 _("Backup file creation failed"));
 		  goto error;
 		}
@@ -965,9 +1028,11 @@ do_replace (GVfsBackend *backend,
 						O_CREAT|O_WRONLY|O_TRUNC, 0);
 	  if (file == NULL)
 	    {
+              int errsv = errno;
+
 	      g_set_error (&error, G_IO_ERROR,
-			   g_io_error_from_errno (errno),
-			   g_strerror (errno));
+			   g_io_error_from_errno (errsv),
+			   "%s", g_strerror (errsv));
 	      goto error;
 	    }
 	}
@@ -1097,10 +1162,12 @@ do_close_write (GVfsBackend *backend,
 						 op_backend->smb_context, handle->backup_uri);
 	  if (res ==  -1)
 	    {
+              int errsv = errno;
+
 	      op_backend->smb_context->unlink (op_backend->smb_context, handle->tmp_uri);
 	      g_vfs_job_failed (G_VFS_JOB (job),
 				G_IO_ERROR, G_IO_ERROR_CANT_CREATE_BACKUP,
-				_("Backup file creation failed: %d"), errno);
+				_("Backup file creation failed: %s"), g_strerror (errsv));
 	      goto out;
 	    }
 	}
@@ -1144,6 +1211,8 @@ set_info_from_stat (GVfsBackendSmb *backend,
   char *content_type;
   char *display_name;
 
+  if (basename) 
+    g_file_info_set_name (info, basename);
 
   if (basename != NULL &&
       g_file_attribute_matcher_matches (matcher,
@@ -1407,10 +1476,12 @@ do_enumerate (GVfsBackend *backend,
 
   if (dir == NULL)
     {
+      int errsv = errno;
+
       error = NULL;
       g_set_error (&error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
-		   g_strerror (errno));
+		   g_io_error_from_errno (errsv),
+		   "%s", g_strerror (errsv));
       goto error;
     }
 
@@ -1461,8 +1532,6 @@ do_enumerate (GVfsBackend *backend,
 		  if (stat_res == 0)
 		    {
 		      info = g_file_info_new ();
-		      g_file_info_set_name (info, dirp->name);
-		      
 		      set_info_from_stat (op_backend, info, &st, dirp->name, matcher);
 		      files = g_list_prepend (files, info);
 		    }
@@ -1507,7 +1576,7 @@ do_set_display_name (GVfsBackend *backend,
   char *dirname, *new_path;
   int res, errsv;
 
-  dirname = g_path_get_basename (filename);
+  dirname = g_path_get_dirname (filename);
 
   /* TODO: display name is in utf8, atm we assume libsmb uris
      are in utf8, but this might not be true if the user changed
@@ -1550,11 +1619,13 @@ do_delete (GVfsBackend *backend,
   res = op_backend->smb_context->stat (op_backend->smb_context, uri, &statbuf);
   if (res == -1)
     {
+      errsv = errno;
+
       g_vfs_job_failed (G_VFS_JOB (job),
 			G_IO_ERROR,
-			g_io_error_from_errno (errno),
+			g_io_error_from_errno (errsv),
 			_("Error deleting file: %s"),
-			g_strerror (errno));
+			g_strerror (errsv));
       g_free (uri);
       return;
     }
@@ -1612,11 +1683,13 @@ do_move (GVfsBackend *backend,
   res = op_backend->smb_context->stat (op_backend->smb_context, source_uri, &statbuf);
   if (res == -1)
     {
+      errsv = errno;
+
       g_vfs_job_failed (G_VFS_JOB (job),
 			G_IO_ERROR,
-			g_io_error_from_errno (errno),
+			g_io_error_from_errno (errsv),
 			_("Error moving file: %s"),
-			g_strerror (errno));
+			g_strerror (errsv));
       g_free (source_uri);
       return;
     }
@@ -1724,6 +1797,9 @@ g_vfs_backend_smb_class_init (GVfsBackendSmbClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GVfsBackendClass *backend_class = G_VFS_BACKEND_CLASS (klass);
+#ifdef HAVE_GCONF
+  GConfClient *gclient;
+#endif
   
   gobject_class->finalize = g_vfs_backend_smb_finalize;
 
@@ -1747,4 +1823,23 @@ g_vfs_backend_smb_class_init (GVfsBackendSmbClass *klass)
   backend_class->make_directory = do_make_directory;
   backend_class->move = do_move;
   backend_class->try_query_settable_attributes = try_query_settable_attributes;
+
+#ifdef HAVE_GCONF
+  gclient = gconf_client_get_default ();
+  if (gclient)
+    {
+      char *workgroup;
+      
+      workgroup = gconf_client_get_string (gclient, 
+					   PATH_GCONF_GNOME_VFS_SMB_WORKGROUP, NULL);
+
+      if (workgroup && workgroup[0])
+	default_workgroup = workgroup;
+      else
+	g_free (workgroup);
+      
+      g_object_unref (gclient);
+    }
+#endif
+
 }
