@@ -99,6 +99,7 @@ struct _GVfsBackendTrash
 
   /* This is only set on the main thread */
   GList *top_files; /* Files in toplevel dir */
+  guint num_top_files;
 
   /* All these are protected by the root_monitor lock */
   GVfsMonitor *file_vfs_monitor;
@@ -112,7 +113,7 @@ struct _GVfsBackendTrash
 
 G_LOCK_DEFINE_STATIC(root_monitor);
 
-G_DEFINE_TYPE (GVfsBackendTrash, g_vfs_backend_trash, G_VFS_TYPE_BACKEND);
+G_DEFINE_TYPE (GVfsBackendTrash, g_vfs_backend_trash, G_VFS_TYPE_BACKEND)
 
 static void   schedule_update_trash_files (GVfsBackendTrash *backend,
                                            gboolean          update_trash_dirs);
@@ -200,8 +201,9 @@ unescape_pathname (const char *escaped_dir, int len)
     {
       char *trashname;
       trashname = g_strndup (escaped_dir + 1, len - 1);
-      return g_build_filename (g_get_user_data_dir (), "Trash", trashname, NULL);
+      dir = g_build_filename (g_get_user_data_dir (), "Trash", trashname, NULL);
       g_free (trashname);
+      return dir;
     }
   
   dir = g_malloc (len + 1 + 1);
@@ -219,7 +221,7 @@ unescape_pathname (const char *escaped_dir, int len)
             *d++ = '_';
           else
             {
-              c = *(p+1);
+              c = *p;
               if (c == '_')
                 {
                   p++;
@@ -252,7 +254,7 @@ get_top_dir_for_trash_dir (const char *trash_dir)
       return g_path_get_dirname (trash_dir);
     }
   
-  user_trash_basename =  g_strdup_printf (".Trash-%d", getuid());
+  user_trash_basename =  g_strdup_printf (".Trash-%u", getuid());
   if (strcmp (basename, user_trash_basename) == 0)
     {
       g_free (user_trash_basename);
@@ -261,7 +263,7 @@ get_top_dir_for_trash_dir (const char *trash_dir)
     }
   g_free (user_trash_basename);
 
-  user_sys_dir = g_strdup_printf ("%d", getuid());
+  user_sys_dir = g_strdup_printf ("%u", getuid());
   if (strcmp (basename, user_sys_dir) == 0)
     {
       g_free (user_sys_dir);
@@ -346,7 +348,7 @@ check_topdir (const char *topdir)
       statbuf.st_mode & S_ISVTX)
     {
       /* We have a valid sysadmin .Trash dir, look for uid subdir */
-      sysadmin_dir_uid = g_strdup_printf ("%s/%d", sysadmin_dir, getuid());
+      sysadmin_dir_uid = g_strdup_printf ("%s/%u", sysadmin_dir, getuid());
       
       if (lstat (sysadmin_dir_uid, &statbuf) == 0 &&
           S_ISDIR (statbuf.st_mode) &&
@@ -362,7 +364,7 @@ check_topdir (const char *topdir)
     }
   g_free (sysadmin_dir);
 
-  user_trash_basename =  g_strdup_printf (".Trash-%d", getuid());
+  user_trash_basename =  g_strdup_printf (".Trash-%u", getuid());
   user_trash = g_build_filename (topdir, user_trash_basename, NULL);
   g_free (user_trash_basename);
   
@@ -552,7 +554,7 @@ list_trash_dirs (void)
 
       if (info & HAS_SYSTEM_DIR)
         {
-          basename = g_strdup_printf ("%d", getuid());
+          basename = g_strdup_printf ("%u", getuid());
           trashdir = g_build_filename (topdir, ".Trash", basename, NULL);
           g_free (basename);
           dirs = g_list_prepend (dirs, trashdir);
@@ -560,7 +562,7 @@ list_trash_dirs (void)
       
       if (info & HAS_USER_DIR)
         {
-          basename = g_strdup_printf (".Trash-%d", getuid());
+          basename = g_strdup_printf (".Trash-%u", getuid());
           trashdir = g_build_filename (topdir, basename, NULL);
           g_free (basename);
           dirs = g_list_prepend (dirs, trashdir);
@@ -594,7 +596,7 @@ g_vfs_backend_trash_init (GVfsBackendTrash *trash_backend)
   GMountSpec *mount_spec;
 
   /* translators: This is the name of the backend */
-  g_vfs_backend_set_display_name (backend, _("Trashcan"));
+  g_vfs_backend_set_display_name (backend, _("Trash"));
   g_vfs_backend_set_icon_name (backend, "user-trash");
   g_vfs_backend_set_user_visible (backend, FALSE);  
 
@@ -614,6 +616,7 @@ do_mount (GVfsBackend *backend,
   GList *names;
 
   names = enumerate_root (backend, NULL);
+  trash_backend->num_top_files = g_list_length (names);
   trash_backend->top_files = g_list_sort (names, (GCompareFunc)strcmp);
   
   g_vfs_job_succeeded (G_VFS_JOB (job));
@@ -758,18 +761,23 @@ typedef struct {
 static gboolean
 set_trash_files (gpointer _data)
 {
+  GVfsMonitor *vfs_monitor, *file_vfs_monitor;
   GVfsBackendTrash *trash_backend;
   SetHasTrashFilesData *data = _data;
-  GList *new, *old, *l;
-  int cmp;
-  char *name;
-  GList *added;
-  GList *removed;
-  GVfsMonitor *vfs_monitor, *file_vfs_monitor;
+  GList *new_list, *old_list;
+  guint old_count, new_count;
 
   trash_backend = G_VFS_BACKEND_TRASH (data->backend);
-  
-  data->names = g_list_sort (data->names, (GCompareFunc)strcmp);
+  new_list = g_list_sort (data->names, (GCompareFunc)strcmp);
+  g_slice_free (SetHasTrashFilesData, data);
+
+  old_list = trash_backend->top_files;
+  old_count = trash_backend->num_top_files;
+
+  /* do the replacement now, before we send notifications.  */
+  new_count = g_list_length (new_list);
+  trash_backend->num_top_files = new_count;
+  trash_backend->top_files = new_list;
  
   G_LOCK (root_monitor);
   vfs_monitor = NULL;
@@ -783,11 +791,16 @@ set_trash_files (gpointer _data)
 
   if (vfs_monitor)
     {
+      GList *added, *removed;
+      GList *new, *old, *l;
+      char *name;
+      int cmp;
+
       added = NULL;
       removed = NULL;
-      
-      old = trash_backend->top_files;
-      new = data->names;
+
+      new = new_list;
+      old = old_list;
       
       while (new != NULL || old != NULL)
         {
@@ -844,10 +857,10 @@ set_trash_files (gpointer _data)
         }
       g_list_free (added);
       
-      if ((trash_backend->top_files == NULL && data->names != NULL) ||
-          (trash_backend->top_files != NULL && data->names == NULL))
+      if (new_count != old_count)
         {
-          /* "fullness" changed => icon change */
+          /* trash::item-count changed. */
+          /* icon change only ever occurs when item-count also changes */
           g_vfs_monitor_emit_event (vfs_monitor,
                                     G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED,
                                     "/",
@@ -859,10 +872,10 @@ set_trash_files (gpointer _data)
 
   if (file_vfs_monitor)
     {
-      if ((trash_backend->top_files == NULL && data->names != NULL) ||
-          (trash_backend->top_files != NULL && data->names == NULL))
+      if (new_count != old_count)
         {
-          /* "fullness" changed => icon change */
+          /* trash::item-count changed. */
+          /* icon change only ever occurs when item-count also changes */
           g_vfs_monitor_emit_event (file_vfs_monitor,
                                     G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED,
                                     "/",
@@ -872,12 +885,8 @@ set_trash_files (gpointer _data)
       g_object_unref (file_vfs_monitor);
     }
 
-  g_list_foreach (trash_backend->top_files, (GFunc)g_free, NULL);
-  g_list_free (trash_backend->top_files);
-
-  trash_backend->top_files = data->names;
-  
-  g_free (data);
+  g_list_foreach (old_list, (GFunc)g_free, NULL);
+  g_list_free (old_list);
   
   return FALSE;
 }
@@ -887,7 +896,7 @@ queue_set_trash_files (GVfsBackend *backend,
 {
   SetHasTrashFilesData *data;
 
-  data = g_new (SetHasTrashFilesData, 1);
+  data = g_slice_new (SetHasTrashFilesData);
   data->backend = backend;
   data->names = names;
   g_idle_add (set_trash_files, data);
@@ -905,7 +914,7 @@ add_extra_trash_info (GFileInfo *file_info,
   char *orig_path, *orig_path_key, *orig_path_unescaped, *date;
   GKeyFile *keyfile;
   char *display_name;
-
+  char *desc;
 
   /* Override all writability */
   g_file_info_set_attribute_boolean (file_info,
@@ -917,6 +926,9 @@ add_extra_trash_info (GFileInfo *file_info,
   g_file_info_set_attribute_boolean (file_info,
                                      G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME,
                                      FALSE);
+  g_file_info_set_attribute_boolean (file_info,
+                                     G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH,
+                                     FALSE);
   
   /* But we can delete */
   g_file_info_set_attribute_boolean (file_info,
@@ -947,8 +959,18 @@ add_extra_trash_info (GFileInfo *file_info,
                   char *p = display_name;
                   display_name = g_strconcat (display_name, _(" (invalid encoding)"), NULL);
                   g_free (p);
+                  
+                  g_file_info_set_attribute_string (file_info, G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME, NULL);
                 }
+              else
+                g_file_info_set_attribute_string (file_info, G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME, display_name);
+                
               g_file_info_set_display_name (file_info, display_name);
+
+              desc = g_strdup_printf (_("%s (in trash)"), display_name);
+              g_file_info_set_attribute_string (file_info, G_FILE_ATTRIBUTE_STANDARD_DESCRIPTION, desc);
+              g_free (desc);
+              
               g_free (display_name);
               
               
@@ -1097,7 +1119,6 @@ do_enumerate (GVfsBackend *backend,
       GFile *file;
       GFileEnumerator *enumerator;
       GFileInfo *info;
-      const char *name;
       char *dir;
       GError *error;
 
@@ -1121,8 +1142,6 @@ do_enumerate (GVfsBackend *backend,
                                                       G_VFS_JOB (job)->cancellable,
                                                       NULL)) != NULL)
             {
-              name = g_file_info_get_name (info);
-              
               g_vfs_job_enumerate_add_info   (job, info);
               g_object_unref (info);
             }
@@ -1160,19 +1179,23 @@ do_query_info (GVfsBackend *backend,
 
   if (!decode_path (filename, &trashdir, &trashfile, &relative_path, &topdir))
     {
+      GVfsBackendTrash *trash_backend = G_VFS_BACKEND_TRASH (backend);
+
       /* The trash:/// root */
       g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
-      g_file_info_set_display_name (info, _("Trashcan"));
+      g_file_info_set_name (info, "/");
+      /* Translators: this is the display name of the backend */
+      g_file_info_set_display_name (info, _("Trash"));
       g_file_info_set_content_type (info, "inode/directory");
 
-      if (G_VFS_BACKEND_TRASH (backend)->top_files != NULL)
+      if (trash_backend->top_files != NULL)
         icon = g_themed_icon_new ("user-trash-full");
       else
         icon = g_themed_icon_new ("user-trash");
         
       g_file_info_set_icon (info, icon);
       g_object_unref (icon);
-      
+     
       g_file_info_set_attribute_boolean (info,
                                          G_FILE_ATTRIBUTE_STANDARD_IS_VIRTUAL,
                                          TRUE);
@@ -1188,9 +1211,15 @@ do_query_info (GVfsBackend *backend,
       g_file_info_set_attribute_boolean (info,
                                          G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE,
                                          FALSE);
+      g_file_info_set_attribute_boolean (info,
+                                         G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH,
+                                         FALSE);
       g_file_info_set_attribute_boolean (info,
                                          G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME,
                                          FALSE);
+      g_file_info_set_attribute_uint32 (info,
+                                        G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT,
+                                        trash_backend->num_top_files);
       
       g_vfs_job_succeeded (G_VFS_JOB (job));
     }
@@ -1201,6 +1230,7 @@ do_query_info (GVfsBackend *backend,
       char *path;
       GError *error; 
       char *info_dir;
+      char *basename;
      
       path = g_build_filename (trashdir, "files", trashfile, relative_path, NULL);
       file = g_file_new_for_path (path);
@@ -1218,6 +1248,10 @@ do_query_info (GVfsBackend *backend,
         {
           g_file_info_copy_into (local_info, info);
 
+          basename = g_path_get_basename (filename);
+          g_file_info_set_name (info, basename);
+          g_free (basename);
+      
           info_dir = g_build_filename (trashdir, "info", NULL);
           add_extra_trash_info (info,
                                 topdir,
